# coding: utf-8

"""
    Weheat Backend

    This is the backend for the Weheat project

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr

class RawHeatPumpLogDto(BaseModel):
    """
    Raw heat pump log as given by a heat pump  (Required star states that it will always be there for any roll, otherwise it is filtered for customers)  # noqa: E501
    """
    heat_pump_id: StrictStr = Field(..., alias="heatPumpId", description="Identifier of the Heat Pump")
    timestamp: datetime = Field(..., description="Timestamp of when this was logged to the server")
    firmware_revision: Optional[StrictInt] = Field(None, alias="firmwareRevision", description="Version of heat pump logs (should be 1026)")
    packet_counter: Optional[StrictInt] = Field(None, alias="packetCounter", description="Packet counter of inside the control board")
    unix_time_mcu: Optional[StrictInt] = Field(None, alias="unixTimeMcu", description="Unix time inside the control board at the time it send this log")
    state: Optional[StrictInt] = Field(None, description="Current heat pump state (which is decoded in the view)")
    rpm_ctrl: Optional[StrictInt] = Field(None, alias="rpmCtrl", description="Current mode of RPM control (which is decoded in the view)")
    error: Optional[StrictInt] = Field(None, description="Current state of error in the heat pump (which is decoded in the view)")
    error_decoded_dtc_none: Optional[StrictBool] = Field(None, alias="errorDecodedDtcNone", description="None state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)")
    error_decoded_dtc_continue: Optional[StrictBool] = Field(None, alias="errorDecodedDtcContinue", description="DTC continue state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)")
    error_decoded_dtc_compressor_off: Optional[StrictBool] = Field(None, alias="errorDecodedDtcCompressorOff", description="DTC off state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)")
    error_decoded_dtc_defrost_forbidden: Optional[StrictBool] = Field(None, alias="errorDecodedDtcDefrostForbidden", description="DTC defrost forbidden state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)")
    error_decoded_dtc_request_service: Optional[StrictBool] = Field(None, alias="errorDecodedDtcRequestService", description="DTC request service state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)")
    error_decoded_dtc_use_heating_curve: Optional[StrictBool] = Field(None, alias="errorDecodedDtcUseHeatingCurve", description="DTC use heating curve state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)")
    error_decoded_dtc_dhw_forbidden: Optional[StrictBool] = Field(None, alias="errorDecodedDtcDhwForbidden", description="DTC use heating curve state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)")
    error_decoded_dtc_error: Optional[StrictBool] = Field(None, alias="errorDecodedDtcError", description="DTC use heating curve state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)")
    error_decoded_dtc_inactive: Optional[StrictBool] = Field(None, alias="errorDecodedDtcInactive", description="DTC Inactive state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)")
    heatsink: Optional[StrictInt] = Field(None, description="Raw state of the where the heat pump is dumping it's heat")
    si5: Optional[StrictInt] = Field(None, description="Raw state of the heat pump")
    e80: Optional[StrictInt] = Field(None, description="Raw error code state 80 of the heat pump")
    e81: Optional[StrictInt] = Field(None, description="Raw error code state 81 of the heat pump")
    e84: Optional[StrictInt] = Field(None, description="Raw error code state 84 of the heat pump")
    e85: Optional[StrictInt] = Field(None, description="Raw error code state 85 of the heat pump")
    control_bridge_status: Optional[StrictInt] = Field(None, alias="controlBridgeStatus", description="Raw control bridge status of the heat pump (bitwise encoded)")
    control_bridge_status_decoded_water_pump: Optional[StrictBool] = Field(None, alias="controlBridgeStatusDecodedWaterPump", description="Water pump (requested) on/off flag of the control bridge status (0x01)")
    control_bridge_status_decoded_dhw_valve: Optional[StrictBool] = Field(None, alias="controlBridgeStatusDecodedDhwValve", description="DhwValve (requested) open/closed flag of the control bridge status (0x02) (DHW Only)")
    control_bridge_status_decoded_gas_boiler: Optional[StrictBool] = Field(None, alias="controlBridgeStatusDecodedGasBoiler", description="Gas boiler assistance (requested) on/off flag of the control bridge status (0x04) (Hybrid Only)")
    control_bridge_status_decoded_electric_heater: Optional[StrictBool] = Field(None, alias="controlBridgeStatusDecodedElectricHeater", description="Electric heater assistance (requested) on/off flag of the control bridge status (0x08) (DHW Only)")
    control_bridge_status_decoded_water_pump2: Optional[StrictBool] = Field(None, alias="controlBridgeStatusDecodedWaterPump2", description="Second water pump (requested) on/off flag of the control bridge status (0x010) (DHW Only)")
    input_status: Optional[StrictInt] = Field(None, alias="inputStatus", description="Raw input status of the heat pump")
    current_control_method: Optional[StrictInt] = Field(None, alias="currentControlMethod", description="Raw current control method state of the heat pump")
    signal_strength: Optional[StrictInt] = Field(None, alias="signalStrength", description="Signal strength (rssi) reported by the modem of the control board")
    t1: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="Temperature T1 (TOP) of the DHW sensors")
    t2: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="Temperature T2 (BOTTOM) of the DHW sensors")
    t_spare_ntc: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="tSpareNtc", description="Spare temperature sensor (for DHW)")
    t_board: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="tBoard", description="Temperature of the control board")
    t_air_in: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="tAirIn", description="Temperature of the air going into the heat pump")
    t_air_out: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="tAirOut", description="Temperature of the air going out of the heat pump")
    t_water_in: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="tWaterIn", description="Temperature of the water going into the heat pump")
    t_water_out: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="tWaterOut", description="Temperature of the water going out of the heat pump")
    t_water_house_in: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="tWaterHouseIn", description="Temperature of the water going into the building")
    rpm: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="Rpm of the compressor")
    rpm_limiter: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="rpmLimiter", description="Rpm limit of the compressor")
    rpm_limiter_type: Optional[StrictInt] = Field(None, alias="rpmLimiterType", description="Type of why the rpm is limited")
    p_compressor_in: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="pCompressorIn", description="Pressure of the compressor of the side where air goes into the compressor")
    p_compressor_out: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="pCompressorOut", description="Pressure of the compressor of the side where air goes out of the compressor")
    p_compressor_in_target: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="pCompressorInTarget", description="Target pressure of the compressor of the side where air goes into the compressor")
    t_inverter: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="tInverter", description="Temperature of the inverter")
    t_compressor_in: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="tCompressorIn", description="Temperature of the compressor of the side where air goes into the compressor")
    t_compressor_out: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="tCompressorOut", description="Temperature of the compressor of the side where air goes out of the compressor")
    t_compressor_in_transient: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="tCompressorInTransient", description="Temperature transient of the compressor of the side where air goes into the compressor")
    t_compressor_out_transient: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="tCompressorOutTransient", description="Temperature transient of the compressor of the side where air goes out of the compressor")
    delta_t_compressor_in_superheat: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="deltaTCompressorInSuperheat", description="Difference in temperature for superheat based on the compressor temperatures")
    compressor_power_low_accuracy: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="compressorPowerLowAccuracy", description="Power used by the compressor (low accuracy)")
    fan: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="State of the fan")
    fan_power: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="fanPower", description="Power used by the fan")
    valve: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="Position value of the valve")
    exv_flow_step_gain: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="exvFlowStepGain")
    cm_mass_flow: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="cmMassFlow", description="Flow of the water going through the heat pump")
    cm_mass_power_in: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="cmMassPowerIn", description="Power going into the heat pump")
    cm_mass_power_out: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="cmMassPowerOut", description="Power going out of the heat pump (in the form of usable heat)")
    p_requested: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="pRequested", description="Power requested by the heat pump")
    power_error_integral: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="powerErrorIntegral", description="Current value of the power error integral")
    on_off_thermostat_state: Optional[StrictInt] = Field(None, alias="onOffThermostatState", description="State of the on/off thermostat")
    thermostat_status: Optional[StrictInt] = Field(None, alias="thermostatStatus", description="Status of the OpenTherm thermostat")
    t_room: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="tRoom", description="Temperature of the room given by the OpenTherm thermostat")
    t_room_target: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="tRoomTarget", description="Target temperature of the room given by the OpenTherm thermostat")
    t_thermostat_setpoint: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="tThermostatSetpoint", description="Setpoint temperature of the OpenTherm thermostat OR the water setpoint of the heat pump")
    ot_boiler_status: Optional[StrictInt] = Field(None, alias="otBoilerStatus", description="Status of the OpenTherm boiler")
    ot_boiler_feed_temperature: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="otBoilerFeedTemperature", description="Feed temperature of the OpenTherm boiler")
    ot_boiler_return_temperature: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="otBoilerReturnTemperature", description="Return temperature of the OpenTherm boiler")
    interval: StrictInt = Field(..., description="Interval for this log in seconds")
    __properties = ["heatPumpId", "timestamp", "firmwareRevision", "packetCounter", "unixTimeMcu", "state", "rpmCtrl", "error", "errorDecodedDtcNone", "errorDecodedDtcContinue", "errorDecodedDtcCompressorOff", "errorDecodedDtcDefrostForbidden", "errorDecodedDtcRequestService", "errorDecodedDtcUseHeatingCurve", "errorDecodedDtcDhwForbidden", "errorDecodedDtcError", "errorDecodedDtcInactive", "heatsink", "si5", "e80", "e81", "e84", "e85", "controlBridgeStatus", "controlBridgeStatusDecodedWaterPump", "controlBridgeStatusDecodedDhwValve", "controlBridgeStatusDecodedGasBoiler", "controlBridgeStatusDecodedElectricHeater", "controlBridgeStatusDecodedWaterPump2", "inputStatus", "currentControlMethod", "signalStrength", "t1", "t2", "tSpareNtc", "tBoard", "tAirIn", "tAirOut", "tWaterIn", "tWaterOut", "tWaterHouseIn", "rpm", "rpmLimiter", "rpmLimiterType", "pCompressorIn", "pCompressorOut", "pCompressorInTarget", "tInverter", "tCompressorIn", "tCompressorOut", "tCompressorInTransient", "tCompressorOutTransient", "deltaTCompressorInSuperheat", "compressorPowerLowAccuracy", "fan", "fanPower", "valve", "exvFlowStepGain", "cmMassFlow", "cmMassPowerIn", "cmMassPowerOut", "pRequested", "powerErrorIntegral", "onOffThermostatState", "thermostatStatus", "tRoom", "tRoomTarget", "tThermostatSetpoint", "otBoilerStatus", "otBoilerFeedTemperature", "otBoilerReturnTemperature", "interval"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RawHeatPumpLogDto:
        """Create an instance of RawHeatPumpLogDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if firmware_revision (nullable) is None
        # and __fields_set__ contains the field
        if self.firmware_revision is None and "firmware_revision" in self.__fields_set__:
            _dict['firmwareRevision'] = None

        # set to None if packet_counter (nullable) is None
        # and __fields_set__ contains the field
        if self.packet_counter is None and "packet_counter" in self.__fields_set__:
            _dict['packetCounter'] = None

        # set to None if unix_time_mcu (nullable) is None
        # and __fields_set__ contains the field
        if self.unix_time_mcu is None and "unix_time_mcu" in self.__fields_set__:
            _dict['unixTimeMcu'] = None

        # set to None if error (nullable) is None
        # and __fields_set__ contains the field
        if self.error is None and "error" in self.__fields_set__:
            _dict['error'] = None

        # set to None if error_decoded_dtc_none (nullable) is None
        # and __fields_set__ contains the field
        if self.error_decoded_dtc_none is None and "error_decoded_dtc_none" in self.__fields_set__:
            _dict['errorDecodedDtcNone'] = None

        # set to None if error_decoded_dtc_continue (nullable) is None
        # and __fields_set__ contains the field
        if self.error_decoded_dtc_continue is None and "error_decoded_dtc_continue" in self.__fields_set__:
            _dict['errorDecodedDtcContinue'] = None

        # set to None if error_decoded_dtc_compressor_off (nullable) is None
        # and __fields_set__ contains the field
        if self.error_decoded_dtc_compressor_off is None and "error_decoded_dtc_compressor_off" in self.__fields_set__:
            _dict['errorDecodedDtcCompressorOff'] = None

        # set to None if error_decoded_dtc_defrost_forbidden (nullable) is None
        # and __fields_set__ contains the field
        if self.error_decoded_dtc_defrost_forbidden is None and "error_decoded_dtc_defrost_forbidden" in self.__fields_set__:
            _dict['errorDecodedDtcDefrostForbidden'] = None

        # set to None if error_decoded_dtc_request_service (nullable) is None
        # and __fields_set__ contains the field
        if self.error_decoded_dtc_request_service is None and "error_decoded_dtc_request_service" in self.__fields_set__:
            _dict['errorDecodedDtcRequestService'] = None

        # set to None if error_decoded_dtc_use_heating_curve (nullable) is None
        # and __fields_set__ contains the field
        if self.error_decoded_dtc_use_heating_curve is None and "error_decoded_dtc_use_heating_curve" in self.__fields_set__:
            _dict['errorDecodedDtcUseHeatingCurve'] = None

        # set to None if error_decoded_dtc_dhw_forbidden (nullable) is None
        # and __fields_set__ contains the field
        if self.error_decoded_dtc_dhw_forbidden is None and "error_decoded_dtc_dhw_forbidden" in self.__fields_set__:
            _dict['errorDecodedDtcDhwForbidden'] = None

        # set to None if error_decoded_dtc_error (nullable) is None
        # and __fields_set__ contains the field
        if self.error_decoded_dtc_error is None and "error_decoded_dtc_error" in self.__fields_set__:
            _dict['errorDecodedDtcError'] = None

        # set to None if error_decoded_dtc_inactive (nullable) is None
        # and __fields_set__ contains the field
        if self.error_decoded_dtc_inactive is None and "error_decoded_dtc_inactive" in self.__fields_set__:
            _dict['errorDecodedDtcInactive'] = None

        # set to None if heatsink (nullable) is None
        # and __fields_set__ contains the field
        if self.heatsink is None and "heatsink" in self.__fields_set__:
            _dict['heatsink'] = None

        # set to None if si5 (nullable) is None
        # and __fields_set__ contains the field
        if self.si5 is None and "si5" in self.__fields_set__:
            _dict['si5'] = None

        # set to None if e80 (nullable) is None
        # and __fields_set__ contains the field
        if self.e80 is None and "e80" in self.__fields_set__:
            _dict['e80'] = None

        # set to None if e81 (nullable) is None
        # and __fields_set__ contains the field
        if self.e81 is None and "e81" in self.__fields_set__:
            _dict['e81'] = None

        # set to None if e84 (nullable) is None
        # and __fields_set__ contains the field
        if self.e84 is None and "e84" in self.__fields_set__:
            _dict['e84'] = None

        # set to None if e85 (nullable) is None
        # and __fields_set__ contains the field
        if self.e85 is None and "e85" in self.__fields_set__:
            _dict['e85'] = None

        # set to None if control_bridge_status (nullable) is None
        # and __fields_set__ contains the field
        if self.control_bridge_status is None and "control_bridge_status" in self.__fields_set__:
            _dict['controlBridgeStatus'] = None

        # set to None if control_bridge_status_decoded_water_pump (nullable) is None
        # and __fields_set__ contains the field
        if self.control_bridge_status_decoded_water_pump is None and "control_bridge_status_decoded_water_pump" in self.__fields_set__:
            _dict['controlBridgeStatusDecodedWaterPump'] = None

        # set to None if control_bridge_status_decoded_dhw_valve (nullable) is None
        # and __fields_set__ contains the field
        if self.control_bridge_status_decoded_dhw_valve is None and "control_bridge_status_decoded_dhw_valve" in self.__fields_set__:
            _dict['controlBridgeStatusDecodedDhwValve'] = None

        # set to None if control_bridge_status_decoded_gas_boiler (nullable) is None
        # and __fields_set__ contains the field
        if self.control_bridge_status_decoded_gas_boiler is None and "control_bridge_status_decoded_gas_boiler" in self.__fields_set__:
            _dict['controlBridgeStatusDecodedGasBoiler'] = None

        # set to None if control_bridge_status_decoded_electric_heater (nullable) is None
        # and __fields_set__ contains the field
        if self.control_bridge_status_decoded_electric_heater is None and "control_bridge_status_decoded_electric_heater" in self.__fields_set__:
            _dict['controlBridgeStatusDecodedElectricHeater'] = None

        # set to None if control_bridge_status_decoded_water_pump2 (nullable) is None
        # and __fields_set__ contains the field
        if self.control_bridge_status_decoded_water_pump2 is None and "control_bridge_status_decoded_water_pump2" in self.__fields_set__:
            _dict['controlBridgeStatusDecodedWaterPump2'] = None

        # set to None if input_status (nullable) is None
        # and __fields_set__ contains the field
        if self.input_status is None and "input_status" in self.__fields_set__:
            _dict['inputStatus'] = None

        # set to None if current_control_method (nullable) is None
        # and __fields_set__ contains the field
        if self.current_control_method is None and "current_control_method" in self.__fields_set__:
            _dict['currentControlMethod'] = None

        # set to None if signal_strength (nullable) is None
        # and __fields_set__ contains the field
        if self.signal_strength is None and "signal_strength" in self.__fields_set__:
            _dict['signalStrength'] = None

        # set to None if t1 (nullable) is None
        # and __fields_set__ contains the field
        if self.t1 is None and "t1" in self.__fields_set__:
            _dict['t1'] = None

        # set to None if t2 (nullable) is None
        # and __fields_set__ contains the field
        if self.t2 is None and "t2" in self.__fields_set__:
            _dict['t2'] = None

        # set to None if t_spare_ntc (nullable) is None
        # and __fields_set__ contains the field
        if self.t_spare_ntc is None and "t_spare_ntc" in self.__fields_set__:
            _dict['tSpareNtc'] = None

        # set to None if t_board (nullable) is None
        # and __fields_set__ contains the field
        if self.t_board is None and "t_board" in self.__fields_set__:
            _dict['tBoard'] = None

        # set to None if rpm (nullable) is None
        # and __fields_set__ contains the field
        if self.rpm is None and "rpm" in self.__fields_set__:
            _dict['rpm'] = None

        # set to None if rpm_limiter (nullable) is None
        # and __fields_set__ contains the field
        if self.rpm_limiter is None and "rpm_limiter" in self.__fields_set__:
            _dict['rpmLimiter'] = None

        # set to None if rpm_limiter_type (nullable) is None
        # and __fields_set__ contains the field
        if self.rpm_limiter_type is None and "rpm_limiter_type" in self.__fields_set__:
            _dict['rpmLimiterType'] = None

        # set to None if p_compressor_in (nullable) is None
        # and __fields_set__ contains the field
        if self.p_compressor_in is None and "p_compressor_in" in self.__fields_set__:
            _dict['pCompressorIn'] = None

        # set to None if p_compressor_out (nullable) is None
        # and __fields_set__ contains the field
        if self.p_compressor_out is None and "p_compressor_out" in self.__fields_set__:
            _dict['pCompressorOut'] = None

        # set to None if p_compressor_in_target (nullable) is None
        # and __fields_set__ contains the field
        if self.p_compressor_in_target is None and "p_compressor_in_target" in self.__fields_set__:
            _dict['pCompressorInTarget'] = None

        # set to None if t_inverter (nullable) is None
        # and __fields_set__ contains the field
        if self.t_inverter is None and "t_inverter" in self.__fields_set__:
            _dict['tInverter'] = None

        # set to None if t_compressor_in (nullable) is None
        # and __fields_set__ contains the field
        if self.t_compressor_in is None and "t_compressor_in" in self.__fields_set__:
            _dict['tCompressorIn'] = None

        # set to None if t_compressor_out (nullable) is None
        # and __fields_set__ contains the field
        if self.t_compressor_out is None and "t_compressor_out" in self.__fields_set__:
            _dict['tCompressorOut'] = None

        # set to None if t_compressor_in_transient (nullable) is None
        # and __fields_set__ contains the field
        if self.t_compressor_in_transient is None and "t_compressor_in_transient" in self.__fields_set__:
            _dict['tCompressorInTransient'] = None

        # set to None if t_compressor_out_transient (nullable) is None
        # and __fields_set__ contains the field
        if self.t_compressor_out_transient is None and "t_compressor_out_transient" in self.__fields_set__:
            _dict['tCompressorOutTransient'] = None

        # set to None if delta_t_compressor_in_superheat (nullable) is None
        # and __fields_set__ contains the field
        if self.delta_t_compressor_in_superheat is None and "delta_t_compressor_in_superheat" in self.__fields_set__:
            _dict['deltaTCompressorInSuperheat'] = None

        # set to None if compressor_power_low_accuracy (nullable) is None
        # and __fields_set__ contains the field
        if self.compressor_power_low_accuracy is None and "compressor_power_low_accuracy" in self.__fields_set__:
            _dict['compressorPowerLowAccuracy'] = None

        # set to None if fan (nullable) is None
        # and __fields_set__ contains the field
        if self.fan is None and "fan" in self.__fields_set__:
            _dict['fan'] = None

        # set to None if fan_power (nullable) is None
        # and __fields_set__ contains the field
        if self.fan_power is None and "fan_power" in self.__fields_set__:
            _dict['fanPower'] = None

        # set to None if valve (nullable) is None
        # and __fields_set__ contains the field
        if self.valve is None and "valve" in self.__fields_set__:
            _dict['valve'] = None

        # set to None if exv_flow_step_gain (nullable) is None
        # and __fields_set__ contains the field
        if self.exv_flow_step_gain is None and "exv_flow_step_gain" in self.__fields_set__:
            _dict['exvFlowStepGain'] = None

        # set to None if cm_mass_flow (nullable) is None
        # and __fields_set__ contains the field
        if self.cm_mass_flow is None and "cm_mass_flow" in self.__fields_set__:
            _dict['cmMassFlow'] = None

        # set to None if cm_mass_power_in (nullable) is None
        # and __fields_set__ contains the field
        if self.cm_mass_power_in is None and "cm_mass_power_in" in self.__fields_set__:
            _dict['cmMassPowerIn'] = None

        # set to None if cm_mass_power_out (nullable) is None
        # and __fields_set__ contains the field
        if self.cm_mass_power_out is None and "cm_mass_power_out" in self.__fields_set__:
            _dict['cmMassPowerOut'] = None

        # set to None if p_requested (nullable) is None
        # and __fields_set__ contains the field
        if self.p_requested is None and "p_requested" in self.__fields_set__:
            _dict['pRequested'] = None

        # set to None if power_error_integral (nullable) is None
        # and __fields_set__ contains the field
        if self.power_error_integral is None and "power_error_integral" in self.__fields_set__:
            _dict['powerErrorIntegral'] = None

        # set to None if thermostat_status (nullable) is None
        # and __fields_set__ contains the field
        if self.thermostat_status is None and "thermostat_status" in self.__fields_set__:
            _dict['thermostatStatus'] = None

        # set to None if t_room (nullable) is None
        # and __fields_set__ contains the field
        if self.t_room is None and "t_room" in self.__fields_set__:
            _dict['tRoom'] = None

        # set to None if t_room_target (nullable) is None
        # and __fields_set__ contains the field
        if self.t_room_target is None and "t_room_target" in self.__fields_set__:
            _dict['tRoomTarget'] = None

        # set to None if ot_boiler_status (nullable) is None
        # and __fields_set__ contains the field
        if self.ot_boiler_status is None and "ot_boiler_status" in self.__fields_set__:
            _dict['otBoilerStatus'] = None

        # set to None if ot_boiler_feed_temperature (nullable) is None
        # and __fields_set__ contains the field
        if self.ot_boiler_feed_temperature is None and "ot_boiler_feed_temperature" in self.__fields_set__:
            _dict['otBoilerFeedTemperature'] = None

        # set to None if ot_boiler_return_temperature (nullable) is None
        # and __fields_set__ contains the field
        if self.ot_boiler_return_temperature is None and "ot_boiler_return_temperature" in self.__fields_set__:
            _dict['otBoilerReturnTemperature'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RawHeatPumpLogDto:
        """Create an instance of RawHeatPumpLogDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RawHeatPumpLogDto.parse_obj(obj)

        _obj = RawHeatPumpLogDto.parse_obj({
            "heat_pump_id": obj.get("heatPumpId"),
            "timestamp": obj.get("timestamp"),
            "firmware_revision": obj.get("firmwareRevision"),
            "packet_counter": obj.get("packetCounter"),
            "unix_time_mcu": obj.get("unixTimeMcu"),
            "state": obj.get("state"),
            "rpm_ctrl": obj.get("rpmCtrl"),
            "error": obj.get("error"),
            "error_decoded_dtc_none": obj.get("errorDecodedDtcNone"),
            "error_decoded_dtc_continue": obj.get("errorDecodedDtcContinue"),
            "error_decoded_dtc_compressor_off": obj.get("errorDecodedDtcCompressorOff"),
            "error_decoded_dtc_defrost_forbidden": obj.get("errorDecodedDtcDefrostForbidden"),
            "error_decoded_dtc_request_service": obj.get("errorDecodedDtcRequestService"),
            "error_decoded_dtc_use_heating_curve": obj.get("errorDecodedDtcUseHeatingCurve"),
            "error_decoded_dtc_dhw_forbidden": obj.get("errorDecodedDtcDhwForbidden"),
            "error_decoded_dtc_error": obj.get("errorDecodedDtcError"),
            "error_decoded_dtc_inactive": obj.get("errorDecodedDtcInactive"),
            "heatsink": obj.get("heatsink"),
            "si5": obj.get("si5"),
            "e80": obj.get("e80"),
            "e81": obj.get("e81"),
            "e84": obj.get("e84"),
            "e85": obj.get("e85"),
            "control_bridge_status": obj.get("controlBridgeStatus"),
            "control_bridge_status_decoded_water_pump": obj.get("controlBridgeStatusDecodedWaterPump"),
            "control_bridge_status_decoded_dhw_valve": obj.get("controlBridgeStatusDecodedDhwValve"),
            "control_bridge_status_decoded_gas_boiler": obj.get("controlBridgeStatusDecodedGasBoiler"),
            "control_bridge_status_decoded_electric_heater": obj.get("controlBridgeStatusDecodedElectricHeater"),
            "control_bridge_status_decoded_water_pump2": obj.get("controlBridgeStatusDecodedWaterPump2"),
            "input_status": obj.get("inputStatus"),
            "current_control_method": obj.get("currentControlMethod"),
            "signal_strength": obj.get("signalStrength"),
            "t1": obj.get("t1"),
            "t2": obj.get("t2"),
            "t_spare_ntc": obj.get("tSpareNtc"),
            "t_board": obj.get("tBoard"),
            "t_air_in": obj.get("tAirIn"),
            "t_air_out": obj.get("tAirOut"),
            "t_water_in": obj.get("tWaterIn"),
            "t_water_out": obj.get("tWaterOut"),
            "t_water_house_in": obj.get("tWaterHouseIn"),
            "rpm": obj.get("rpm"),
            "rpm_limiter": obj.get("rpmLimiter"),
            "rpm_limiter_type": obj.get("rpmLimiterType"),
            "p_compressor_in": obj.get("pCompressorIn"),
            "p_compressor_out": obj.get("pCompressorOut"),
            "p_compressor_in_target": obj.get("pCompressorInTarget"),
            "t_inverter": obj.get("tInverter"),
            "t_compressor_in": obj.get("tCompressorIn"),
            "t_compressor_out": obj.get("tCompressorOut"),
            "t_compressor_in_transient": obj.get("tCompressorInTransient"),
            "t_compressor_out_transient": obj.get("tCompressorOutTransient"),
            "delta_t_compressor_in_superheat": obj.get("deltaTCompressorInSuperheat"),
            "compressor_power_low_accuracy": obj.get("compressorPowerLowAccuracy"),
            "fan": obj.get("fan"),
            "fan_power": obj.get("fanPower"),
            "valve": obj.get("valve"),
            "exv_flow_step_gain": obj.get("exvFlowStepGain"),
            "cm_mass_flow": obj.get("cmMassFlow"),
            "cm_mass_power_in": obj.get("cmMassPowerIn"),
            "cm_mass_power_out": obj.get("cmMassPowerOut"),
            "p_requested": obj.get("pRequested"),
            "power_error_integral": obj.get("powerErrorIntegral"),
            "on_off_thermostat_state": obj.get("onOffThermostatState"),
            "thermostat_status": obj.get("thermostatStatus"),
            "t_room": obj.get("tRoom"),
            "t_room_target": obj.get("tRoomTarget"),
            "t_thermostat_setpoint": obj.get("tThermostatSetpoint"),
            "ot_boiler_status": obj.get("otBoilerStatus"),
            "ot_boiler_feed_temperature": obj.get("otBoilerFeedTemperature"),
            "ot_boiler_return_temperature": obj.get("otBoilerReturnTemperature"),
            "interval": obj.get("interval")
        })
        return _obj





# coding: utf-8

"""
    Weheat Backend

    This is the backend for the Weheat project

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Optional
from pydantic import BaseModel, Field, StrictStr
from weheat.models.role import Role

class ReadUserDto(BaseModel):
    """
    ReadUserDto
    """
    id: StrictStr = Field(..., description="Identifier of the user")
    first_name: Optional[StrictStr] = Field(None, alias="firstName", description="First name of the user if available")
    last_name: Optional[StrictStr] = Field(None, alias="lastName", description="Last Name of the user if available")
    role: Role = Field(...)
    email: Optional[StrictStr] = Field(None, description="Email address of the user")
    updated_on: datetime = Field(..., alias="updatedOn", description="Timestamp of the last update to the user entry")
    created_on: datetime = Field(..., alias="createdOn", description="Timestamp of the creation of the user entry")
    __properties = ["id", "firstName", "lastName", "role", "email", "updatedOn", "createdOn"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ReadUserDto:
        """Create an instance of ReadUserDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if first_name (nullable) is None
        # and __fields_set__ contains the field
        if self.first_name is None and "first_name" in self.__fields_set__:
            _dict['firstName'] = None

        # set to None if last_name (nullable) is None
        # and __fields_set__ contains the field
        if self.last_name is None and "last_name" in self.__fields_set__:
            _dict['lastName'] = None

        # set to None if email (nullable) is None
        # and __fields_set__ contains the field
        if self.email is None and "email" in self.__fields_set__:
            _dict['email'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ReadUserDto:
        """Create an instance of ReadUserDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ReadUserDto.parse_obj(obj)

        _obj = ReadUserDto.parse_obj({
            "id": obj.get("id"),
            "first_name": obj.get("firstName"),
            "last_name": obj.get("lastName"),
            "role": obj.get("role"),
            "email": obj.get("email"),
            "updated_on": obj.get("updatedOn"),
            "created_on": obj.get("createdOn")
        })
        return _obj





# coding: utf-8

"""
    Weheat Backend

    This is the backend for the Weheat project

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class HeatPumpStatusEnum(int, Enum):
    """
    Last status of the heat pump from the last heat pump log   - Standby (40)   - Heating (70)   - Defrost (90)   - Cooling (130)   - DHW (150)   - Lionelle (160)   - SelfTest (170)  -  ManualControl (180)
    """

    """
    allowed enum values
    """
    NUMBER_40 = 40
    NUMBER_70 = 70
    NUMBER_90 = 90
    NUMBER_130 = 130
    NUMBER_150 = 150
    NUMBER_160 = 160
    NUMBER_170 = 170
    NUMBER_180 = 180

    @classmethod
    def from_json(cls, json_str: str) -> HeatPumpStatusEnum:
        """Create an instance of HeatPumpStatusEnum from a JSON string"""
        return HeatPumpStatusEnum(json.loads(json_str))





from weheat.configuration import Configuration
from weheat.api_client import ApiClient
from weheat.api.user_api import UserApi


async def get_user_id_from_token(api_url: str, access_token: str):
    """ Get the user id from the current logged-in user. """
    try:
        config = Configuration(host=api_url, access_token=access_token)

        with ApiClient(configuration=config) as client:
            response = UserApi(
                client
            ).api_v1_users_me_get_with_http_info(async_req=True).get()
            if response.status_code == 200:
                return response.data.id
    except Exception as e:
        raise e



# coding: utf-8

"""
    Weheat Backend

    This is the backend for the Weheat project

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Optional
from pydantic import BaseModel, Field, StrictStr, constr
from weheat.models.boiler_type import BoilerType
from weheat.models.device_state import DeviceState
from weheat.models.dhw_type import DhwType
from weheat.models.heat_pump_model import HeatPumpModel
from weheat.models.heat_pump_status_enum import HeatPumpStatusEnum
from weheat.models.heat_pump_type import HeatPumpType

class ReadAllHeatPumpDto(BaseModel):
    """
    Heat Pump dto used for the GetAll operation where the firmware version is needed  # noqa: E501
    """
    id: StrictStr = Field(..., description="Identifier of the heat pump")
    control_board_id: StrictStr = Field(..., alias="controlBoardId", description="Identifier of the control board that is connected to the heat pump")
    serial_number: constr(strict=True, min_length=1) = Field(..., alias="serialNumber", description="Serial Number of this heat pump")
    part_number: Optional[StrictStr] = Field(None, alias="partNumber", description="Part Number of this heat pump")
    state: DeviceState = Field(...)
    name: Optional[StrictStr] = Field(None, description="Internal nickname of this heat pump")
    model: Optional[HeatPumpModel] = None
    dhw_type: Optional[DhwType] = Field(None, alias="dhwType")
    boiler_type: Optional[BoilerType] = Field(None, alias="boilerType")
    status: Optional[HeatPumpStatusEnum] = None
    type: Optional[HeatPumpType] = None
    commissioned_at: Optional[datetime] = Field(None, alias="commissionedAt", description="Date when the heat pump was last commissioned (decommissioning sets this to null)")
    firmware_version: Optional[StrictStr] = Field(None, alias="firmwareVersion", description="Firmware version of control board connected to heat pump")
    __properties = ["id", "controlBoardId", "serialNumber", "partNumber", "state", "name", "model", "dhwType", "boilerType", "status", "type", "commissionedAt", "firmwareVersion"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ReadAllHeatPumpDto:
        """Create an instance of ReadAllHeatPumpDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if part_number (nullable) is None
        # and __fields_set__ contains the field
        if self.part_number is None and "part_number" in self.__fields_set__:
            _dict['partNumber'] = None

        # set to None if name (nullable) is None
        # and __fields_set__ contains the field
        if self.name is None and "name" in self.__fields_set__:
            _dict['name'] = None

        # set to None if commissioned_at (nullable) is None
        # and __fields_set__ contains the field
        if self.commissioned_at is None and "commissioned_at" in self.__fields_set__:
            _dict['commissionedAt'] = None

        # set to None if firmware_version (nullable) is None
        # and __fields_set__ contains the field
        if self.firmware_version is None and "firmware_version" in self.__fields_set__:
            _dict['firmwareVersion'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ReadAllHeatPumpDto:
        """Create an instance of ReadAllHeatPumpDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ReadAllHeatPumpDto.parse_obj(obj)

        _obj = ReadAllHeatPumpDto.parse_obj({
            "id": obj.get("id"),
            "control_board_id": obj.get("controlBoardId"),
            "serial_number": obj.get("serialNumber"),
            "part_number": obj.get("partNumber"),
            "state": obj.get("state"),
            "name": obj.get("name"),
            "model": obj.get("model"),
            "dhw_type": obj.get("dhwType"),
            "boiler_type": obj.get("boilerType"),
            "status": obj.get("status"),
            "type": obj.get("type"),
            "commissioned_at": obj.get("commissionedAt"),
            "firmware_version": obj.get("firmwareVersion")
        })
        return _obj





"""Weheat heat pump abstraction from the API."""
from enum import Enum, auto
from weheat.configuration import Configuration
from weheat.api_client import ApiClient
from weheat.api.heat_pump_log_api import HeatPumpLogApi
from weheat.api.energy_log_api import EnergyLogApi
from datetime import datetime, timedelta

# before this date no energy logs are available, so start from this point onwards
START_DATE = datetime(2023, 1, 1, 0, 0, 0)


class HeatPump:
    """Heat pump class representing a heat pump."""
    class State(Enum):
        STANDBY = auto()
        WATER_CHECK = auto()
        HEATING = auto()
        COOLING = auto()
        DHW = auto()
        LEGIONELLA_PREVENTION = auto()
        DEFROSTING = auto()
        SELF_TEST = auto()
        MANUAL_CONTROL = auto()

    def __init__(self, api_url: str, uuid: str) -> None:
        self._api_url = api_url
        self._uuid = uuid
        self._last_log = None
        self._energy_consumption = None

    def get_status(self, access_token: str):
        """Updates the heat pump instance with data from the API."""
        try:
            config = Configuration(host=self._api_url, access_token=access_token)

            with ApiClient(configuration=config) as client:
                response = HeatPumpLogApi(
                    client
                ).api_v1_heat_pumps_heat_pump_id_logs_latest_get_with_http_info(
                    heat_pump_id=self._uuid
                )
                if response.status_code == 200:
                    self._last_log = response.data

                # Also get all energy totals form past years and add them together
                # As end time pick today + 1 day to avoid issues with timezones
                response = EnergyLogApi(client).api_v1_energy_logs_heat_pump_id_get_with_http_info(heat_pump_id=self._uuid,
                                                                                start_time=START_DATE,
                                                                                end_time=datetime.now() + timedelta(days=1),
                                                                                interval='Year')

                if response.status_code == 200:
                    # aggregate the energy consumption
                    self._energy_consumption = 0
                    for year in response.data:
                        self._energy_consumption += year.total_ein_cooling
                        self._energy_consumption += year.total_ein_heating
                        self._energy_consumption += year.total_ein_heating_defrost
                        self._energy_consumption += year.total_ein_dhw
                        self._energy_consumption += year.total_ein_dhw_defrost



        except Exception as e:
            self._last_log = None
            self._energy_consumption = None
            raise e

    def _if_available(self, key):
        """Returns the value from the last logged value if available. None otherwise."""
        if self._last_log is not None and hasattr(self._last_log, key):
            return getattr(self._last_log, key)
        return None

    def __str__(self):
        return f"WeheatHeatPump(uuid={self._uuid}, last update={self._if_available('timestamp')})"

    def __repr__(self):
        return self.__str__()

    @property
    def water_inlet_temperature(self):
        """The heat pump water inlet temperature."""
        return self._if_available("t_water_in")

    @property
    def water_outlet_temperature(self):
        """The heat pump water outlet temperature."""
        return self._if_available("t_water_out")

    @property
    def water_house_in_temperature(self):
        """The water house in temperature."""
        return self._if_available("t_water_house_in")

    @property
    def air_inlet_temperature(self):
        """The heat pump air inlet temperature."""
        return self._if_available("t_air_in")

    @property
    def air_outlet_temperature(self):
        """The heat pump air outlet temperature."""
        return self._if_available("t_air_out")

    @property
    def thermostat_water_setpoint(self):
        """The  thermostat water setpoint."""
        return self._if_available("t_thermostat_setpoint")

    @property
    def thermostat_room_temperature(self):
        """The thermostat current room temperature."""
        return self._if_available("t_room")

    @property
    def thermostat_room_temperature_setpoint(self):
        """The thermostat room temperature setpoint."""
        return self._if_available("t_room_target")

    @property
    def thermostat_on_off_state(self):
        """The thermostat on/off state."""
        return self._if_available("on_off_thermostat_state")

    @property
    def power_input(self):
        """The heat pumps power input."""
        return self._if_available("cm_mass_power_in")

    @property
    def power_output(self):
        """The heat pumps hydraulic output power."""
        return self._if_available("cm_mass_power_out")

    @property
    def dhw_top_temperature(self):
        """The DHW vessel top temperature."""
        return self._if_available("t1")

    @property
    def dhw_bottom_temperature(self):
        """The DHW vessel bottom temperature."""
        return self._if_available("t2")

    @property
    def cop(self):
        """
        Returns the coefficient of performance of the heat pump.
        Note that this is calculated from a singular log entry and might not be accurate when the
        heat pump is changing its output power or switching states
        """
        input = self.power_input
        output = self.power_output
        if input is not None and output is not None and input != 0:
            return output / input

    @property
    def inside_unit_water_pump_state(self):
        """Decoded water pump state."""
        return self._if_available("control_bridge_status_decoded_water_pump")

    @property
    def inside_unit_auxilary_pump_state(self):
        """Decoded auxilary pump state."""
        return self._if_available("control_bridge_status_decoded_water_pump2")

    @property
    def inside_unit_dhw_valve_or_pump_state(self):
        """Decoded DHW valve or pump state."""
        return self._if_available("control_bridge_status_decoded_dhw_valve")

    @property
    def inside_unit_gas_boiler_state(self):
        """Decoded gas boiler state."""
        return self._if_available("control_bridge_status_decoded_gas_boiler")

    @property
    def heat_pump_state(self) -> State | None:
        """The heat pump state."""
        numeric_state = self._if_available("state")
        if numeric_state is None:
            return None

        if numeric_state == 40:
            return self.State.STANDBY
        elif numeric_state == 70:
            return self.State.HEATING
        elif numeric_state == 130:
            return self.State.COOLING
        elif numeric_state == 150:
            return self.State.DHW
        elif numeric_state == 160:
            return self.State.LEGIONELLA_PREVENTION
        elif numeric_state == 170:
            return self.State.SELF_TEST
        elif numeric_state == 180:
            return self.State.MANUAL_CONTROL
        elif numeric_state == 200:
            return self.State.DEFROSTING
        return None

    @property
    def energy_total(self):
        """The total used energy in kWh from 2023 to now."""
        return self._energy_consumption



# coding: utf-8

"""
    Weheat Backend

    This is the backend for the Weheat project

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Optional, Union
from pydantic import BaseModel, Field, StrictFloat, StrictInt, StrictStr

class EnergyViewDto(BaseModel):
    """
    EnergyViewDto
    """
    interval: Optional[StrictStr] = Field(None, description="Interval of this EnergyViewDto (Correct intervals include: \"Hour\", \"Day\", \"Week\", \"Month\", \"Year\")")
    time_bucket: Optional[datetime] = Field(None, alias="timeBucket")
    total_ein_heating: Union[StrictFloat, StrictInt] = Field(..., alias="totalEInHeating", description="Total energy from electricity going into the heat pump for heating for this interval (in kwh)")
    total_ein_dhw: Union[StrictFloat, StrictInt] = Field(..., alias="totalEInDhw", description="Total energy from electricity going into the heat pump for doing DHW for this interval (in kwh)")
    total_ein_heating_defrost: Union[StrictFloat, StrictInt] = Field(..., alias="totalEInHeatingDefrost", description="Total energy from electricity going into the heat pump for defrosting whilst heating for this interval (in kwh)")
    total_ein_dhw_defrost: Union[StrictFloat, StrictInt] = Field(..., alias="totalEInDhwDefrost", description="Total energy from electricity going into the heat pump for defrosting whilst doing DHW for this interval (in kwh)")
    total_ein_cooling: Union[StrictFloat, StrictInt] = Field(..., alias="totalEInCooling", description="Total energy from electricity going into the heat pump for cooling for this interval (in kwh)")
    total_e_out_heating: Union[StrictFloat, StrictInt] = Field(..., alias="totalEOutHeating", description="Total energy from electricity going out of the heat pump for heating for this interval (in kwh)")
    total_e_out_dhw: Union[StrictFloat, StrictInt] = Field(..., alias="totalEOutDhw", description="Total energy from electricity going out of the heat pump for doing DHW for this interval (in kwh)")
    total_e_out_heating_defrost: Union[StrictFloat, StrictInt] = Field(..., alias="totalEOutHeatingDefrost", description="Total energy from electricity going out of the heat pump for defrosting whilst heating for this interval (in kwh)")
    total_e_out_dhw_defrost: Union[StrictFloat, StrictInt] = Field(..., alias="totalEOutDhwDefrost", description="Total energy from electricity going out of the heat pump for defrosting whilst doing DHW for this interval (in kwh)")
    total_e_out_cooling: Union[StrictFloat, StrictInt] = Field(..., alias="totalEOutCooling", description="Total energy from electricity going out of the heat pump for cooling for this interval (in kwh)")
    average_power_ein_heating: Union[StrictFloat, StrictInt] = Field(..., alias="averagePowerEInHeating", description="Average power from electricity going into the heat pump for heating for this interval (in kW)")
    average_power_ein_dhw: Union[StrictFloat, StrictInt] = Field(..., alias="averagePowerEInDhw", description="Average power from electricity going into the heat pump for doing DHW for this interval (in kW)")
    average_power_ein_heating_defrost: Union[StrictFloat, StrictInt] = Field(..., alias="averagePowerEInHeatingDefrost", description="Average power from electricity going into the heat pump for defrosting whilst heating for this interval (in kW)")
    average_power_ein_dhw_defrost: Union[StrictFloat, StrictInt] = Field(..., alias="averagePowerEInDhwDefrost", description="Average power from electricity going into the heat pump for defrosting whilst doing DHW for this interval (in kW)")
    average_power_ein_cooling: Union[StrictFloat, StrictInt] = Field(..., alias="averagePowerEInCooling", description="Average power from electricity going into the heat pump for cooling for this interval (in kW)")
    average_power_e_out_heating: Union[StrictFloat, StrictInt] = Field(..., alias="averagePowerEOutHeating", description="Average power from electricity going out of the heat pump for heating for this interval (in kW)")
    average_power_e_out_dhw: Union[StrictFloat, StrictInt] = Field(..., alias="averagePowerEOutDhw", description="Average power from electricity going out of the heat pump for doing DHW for this interval (in kW)")
    average_power_e_out_heating_defrost: Union[StrictFloat, StrictInt] = Field(..., alias="averagePowerEOutHeatingDefrost", description="Average power from electricity going out of the heat pump for defrosting whilst heating for this interval (in kW)")
    average_power_e_out_dhw_defrost: Union[StrictFloat, StrictInt] = Field(..., alias="averagePowerEOutDhwDefrost", description="Average power from electricity going out of the heat pump for defrosting whilst doing DHW for this interval (in kW)")
    average_power_e_out_cooling: Union[StrictFloat, StrictInt] = Field(..., alias="averagePowerEOutCooling", description="Average power from electricity going out of the heat pump for cooling for this interval (in kW)")
    __properties = ["interval", "timeBucket", "totalEInHeating", "totalEInDhw", "totalEInHeatingDefrost", "totalEInDhwDefrost", "totalEInCooling", "totalEOutHeating", "totalEOutDhw", "totalEOutHeatingDefrost", "totalEOutDhwDefrost", "totalEOutCooling", "averagePowerEInHeating", "averagePowerEInDhw", "averagePowerEInHeatingDefrost", "averagePowerEInDhwDefrost", "averagePowerEInCooling", "averagePowerEOutHeating", "averagePowerEOutDhw", "averagePowerEOutHeatingDefrost", "averagePowerEOutDhwDefrost", "averagePowerEOutCooling"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> EnergyViewDto:
        """Create an instance of EnergyViewDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if interval (nullable) is None
        # and __fields_set__ contains the field
        if self.interval is None and "interval" in self.__fields_set__:
            _dict['interval'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> EnergyViewDto:
        """Create an instance of EnergyViewDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return EnergyViewDto.parse_obj(obj)

        _obj = EnergyViewDto.parse_obj({
            "interval": obj.get("interval"),
            "time_bucket": obj.get("timeBucket"),
            "total_ein_heating": obj.get("totalEInHeating"),
            "total_ein_dhw": obj.get("totalEInDhw"),
            "total_ein_heating_defrost": obj.get("totalEInHeatingDefrost"),
            "total_ein_dhw_defrost": obj.get("totalEInDhwDefrost"),
            "total_ein_cooling": obj.get("totalEInCooling"),
            "total_e_out_heating": obj.get("totalEOutHeating"),
            "total_e_out_dhw": obj.get("totalEOutDhw"),
            "total_e_out_heating_defrost": obj.get("totalEOutHeatingDefrost"),
            "total_e_out_dhw_defrost": obj.get("totalEOutDhwDefrost"),
            "total_e_out_cooling": obj.get("totalEOutCooling"),
            "average_power_ein_heating": obj.get("averagePowerEInHeating"),
            "average_power_ein_dhw": obj.get("averagePowerEInDhw"),
            "average_power_ein_heating_defrost": obj.get("averagePowerEInHeatingDefrost"),
            "average_power_ein_dhw_defrost": obj.get("averagePowerEInDhwDefrost"),
            "average_power_ein_cooling": obj.get("averagePowerEInCooling"),
            "average_power_e_out_heating": obj.get("averagePowerEOutHeating"),
            "average_power_e_out_dhw": obj.get("averagePowerEOutDhw"),
            "average_power_e_out_heating_defrost": obj.get("averagePowerEOutHeatingDefrost"),
            "average_power_e_out_dhw_defrost": obj.get("averagePowerEOutDhwDefrost"),
            "average_power_e_out_cooling": obj.get("averagePowerEOutCooling")
        })
        return _obj





# coding: utf-8

"""
    Weheat Backend

    This is the backend for the Weheat project

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import re  # noqa: F401
import io
import warnings

from pydantic import validate_arguments, ValidationError

from typing_extensions import Annotated
from datetime import datetime

from pydantic import Field, StrictStr

from typing import List, Optional

from weheat.models.energy_view_dto import EnergyViewDto

from weheat.api_client import ApiClient
from weheat.api_response import ApiResponse
from weheat.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class EnergyLogApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def api_v1_energy_logs_heat_pump_id_get(self, heat_pump_id : Annotated[StrictStr, Field(..., description="The id of the heat pump")], start_time : Annotated[Optional[datetime], Field(description="Start time of the interval as a DateTime object in UTC (format: yyyy-MM-dd HH:mm:ss)")] = None, end_time : Annotated[Optional[datetime], Field(description="End time of the interval as a DateTime object in UTC (format: yyyy-MM-dd HH:mm:ss)")] = None, interval : Annotated[Optional[StrictStr], Field(description="Interval granularity of the log data, allowed intervals: \"Hour\", \"Day\", \"Week\", \"Month\", \"Year\"")] = None, x_version : Annotated[Optional[StrictStr], Field(description="Optional version parameter for frontend applications to check if an update / refresh is needed")] = None, **kwargs) -> List[EnergyViewDto]:  # noqa: E501
        """Gets the energy logs of the heat pump in a from start time to end time for a given interval  Correct Intervals include: Hour, Day, Week, Month, Year  Hour logs are rounded to 3 decimals, other logs to 2 decimals  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.api_v1_energy_logs_heat_pump_id_get(heat_pump_id, start_time, end_time, interval, x_version, async_req=True)
        >>> result = thread.get()

        :param heat_pump_id: The id of the heat pump (required)
        :type heat_pump_id: str
        :param start_time: Start time of the interval as a DateTime object in UTC (format: yyyy-MM-dd HH:mm:ss)
        :type start_time: datetime
        :param end_time: End time of the interval as a DateTime object in UTC (format: yyyy-MM-dd HH:mm:ss)
        :type end_time: datetime
        :param interval: Interval granularity of the log data, allowed intervals: \"Hour\", \"Day\", \"Week\", \"Month\", \"Year\"
        :type interval: str
        :param x_version: Optional version parameter for frontend applications to check if an update / refresh is needed
        :type x_version: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: List[EnergyViewDto]
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the api_v1_energy_logs_heat_pump_id_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.api_v1_energy_logs_heat_pump_id_get_with_http_info(heat_pump_id, start_time, end_time, interval, x_version, **kwargs)  # noqa: E501

    @validate_arguments
    def api_v1_energy_logs_heat_pump_id_get_with_http_info(self, heat_pump_id : Annotated[StrictStr, Field(..., description="The id of the heat pump")], start_time : Annotated[Optional[datetime], Field(description="Start time of the interval as a DateTime object in UTC (format: yyyy-MM-dd HH:mm:ss)")] = None, end_time : Annotated[Optional[datetime], Field(description="End time of the interval as a DateTime object in UTC (format: yyyy-MM-dd HH:mm:ss)")] = None, interval : Annotated[Optional[StrictStr], Field(description="Interval granularity of the log data, allowed intervals: \"Hour\", \"Day\", \"Week\", \"Month\", \"Year\"")] = None, x_version : Annotated[Optional[StrictStr], Field(description="Optional version parameter for frontend applications to check if an update / refresh is needed")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Gets the energy logs of the heat pump in a from start time to end time for a given interval  Correct Intervals include: Hour, Day, Week, Month, Year  Hour logs are rounded to 3 decimals, other logs to 2 decimals  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.api_v1_energy_logs_heat_pump_id_get_with_http_info(heat_pump_id, start_time, end_time, interval, x_version, async_req=True)
        >>> result = thread.get()

        :param heat_pump_id: The id of the heat pump (required)
        :type heat_pump_id: str
        :param start_time: Start time of the interval as a DateTime object in UTC (format: yyyy-MM-dd HH:mm:ss)
        :type start_time: datetime
        :param end_time: End time of the interval as a DateTime object in UTC (format: yyyy-MM-dd HH:mm:ss)
        :type end_time: datetime
        :param interval: Interval granularity of the log data, allowed intervals: \"Hour\", \"Day\", \"Week\", \"Month\", \"Year\"
        :type interval: str
        :param x_version: Optional version parameter for frontend applications to check if an update / refresh is needed
        :type x_version: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(List[EnergyViewDto], status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'heat_pump_id',
            'start_time',
            'end_time',
            'interval',
            'x_version'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method api_v1_energy_logs_heat_pump_id_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['heat_pump_id'] is not None:
            _path_params['heatPumpId'] = _params['heat_pump_id']


        # process the query parameters
        _query_params = []
        if _params.get('start_time') is not None:  # noqa: E501
            if isinstance(_params['start_time'], datetime):
                _query_params.append(('startTime', _params['start_time'].strftime(self.api_client.configuration.datetime_format)))
            else:
                _query_params.append(('startTime', _params['start_time']))

        if _params.get('end_time') is not None:  # noqa: E501
            if isinstance(_params['end_time'], datetime):
                _query_params.append(('endTime', _params['end_time'].strftime(self.api_client.configuration.datetime_format)))
            else:
                _query_params.append(('endTime', _params['end_time']))

        if _params.get('interval') is not None:  # noqa: E501
            _query_params.append(('interval', _params['interval']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['x_version'] is not None:
            _header_params['x-version'] = _params['x_version']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['text/plain', 'application/json', 'text/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['oauth2']  # noqa: E501

        _response_types_map = {
            '403': "str",
            '505': None,
            '200': "List[EnergyViewDto]",
            '400': None,
            '500': None,
        }

        return self.api_client.call_api(
            '/api/v1/energy-logs/{heatPumpId}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))
